; From : https://stackoverflow.com/questions/43786251/int-13h-42h-doesnt-load-anything-in-bochs
; Complete Example with Int 13h Extension Checks
; The following code will check that Int 13h extensions are available in the BIOS 
; and will also determine if the drive in DL supports Int 13h extensions. 
; If there is a failure the code will print an appropriate error. 
; A print_string function is used to display strings to the console using BIOS TTY output 
; and a print_hex_word function is provided to display the boot drive number in hexadecimal.

org 0x7c00
bits 16

section .text
boot:
    jmp main
    TIMES 3-($-$$) DB 0x90   ; Support 2 or 3 byte encoded JMPs before BPB.

    ; Dos 4.0 EBPB 1.44MB floppy
    OEMname:           db    "mkfs.fat"  ; mkfs.fat is what OEMname mkdosfs uses
    bytesPerSector:    dw    512
    sectPerCluster:    db    1
    reservedSectors:   dw    1
    numFAT:            db    2
    numRootDirEntries: dw    224
    numSectors:        dw    2880
    mediaType:         db    0xf0
    numFATsectors:     dw    9
    sectorsPerTrack:   dw    18
    numHeads:          dw    2
    numHiddenSectors:  dd    0
    numSectorsHuge:    dd    0
    driveNum:          db    0
    reserved:          db    0
    signature:         db    0x29
    volumeID:          dd    0x2d7e5a1a
    volumeLabel:       db    "NO NAME    "
    fileSysType:       db    "FAT12   "

main:
    cli
    cld                   ; String instructions forward movement

    xor ax, ax
    mov ds, ax
    mov es, ax
    ; Setup 4K stack before this bootloader
    mov ss, ax
    mov sp, 0x7c00

    ; Display a banner to know our bootloader is executing
    mov si, msg_booting
    call print_string

    ; Check that Int 13h Extensions are available
    ; http://www.ctyme.com/intr/rb-0706.htm

    mov ah, 0x41          ; Int 13h/AH=41h: Check if extensions present
    mov bx, 0x55aa
    int 0x13
    jc  ext_drv_none      ; CF set - no extensions available for drive
    cmp bx, 0xaa55        ; Is BX 0xaa55?
    jnz ext_none          ;     If not, int 13h extensions not supported
                          ;     by BIOS at all.

    ; Int 13h extensions supported by BIOS and drive at this point
    ; Load next sectors
    mov si, DAP
    mov ah, 42h
    ; DL didn't changed
    int 13h
    ; Start
    jmp bootend

; Error: BIOS doesn't support Int 13h extensions
ext_none:
    mov si, err_no_extensions
    call print_string
    jmp error_end

; Error: BIOS supports Int 13h extensions but not for drive in DL
ext_drv_none:
    mov si, err_no_drv_ext_support
    call print_string

    ; Print the boot drive number in hex
    xor dh, dh            ; Zero extended drive number to all of DX
    push word 0x00        ; Attribute and page number to write to
    push dx               ; The value to write as hex
    call print_hex_word


error_end:
    cli
.loop:
    hlt
    jmp .loop

; Print 16 bit value passed on stack as first parameter
; in hexadecimal. Use page number and foreground color
; passed in second parameter. This routine will work on 8086+
; processors. This code takes advantage of packed BCD to
; determine the ASCII values to print. This code could have
; used compare and branch to do the same or a translation table.

print_hex_word:
    push bp
    mov bp, sp      ; BP=SP, on 8086 can't use sp in memory operand
    push dx         ; Save all registers we clobber
    push cx
    push bx
    push ax

    mov cx, 0x0404  ; CH = number of nibbles to process = 4 (4*4=16 bits)
                    ; CL = Number of bits to rotate each iteration = 4 (a nibble)
    mov dx, [bp+4]  ; DX = word parameter on stack at [bp+4] to print
    mov bx, [bp+6]  ; BX = page / foreground attr is at [bp+6]

.loop:
    rol dx, cl      ; Roll 4 bits left. Lower nibble is value to print
    mov ax, 0x0e0f  ; AH=0E (BIOS tty print),AL=mask to get lower nibble
    and al, dl      ; AL=copy of lower nibble
    add al, 0x90    ; Work as if we are packed BCD
    daa             ; Decimal adjust after add.
                    ;    If nibble in AL was between 0 and 9, then CF=0 and
                    ;    AL=0x90 to 0x99
                    ;    If nibble in AL was between A and F, then CF=1 and
                    ;    AL=0x00 to 0x05
    adc al, 0x40    ; AL=0xD0 to 0xD9
                    ; or AL=0x41 to 0x46
    daa             ; AL=0x30 to 0x39 (ASCII '0' to '9')
                    ; or AL=0x41 to 0x46 (ASCII 'A' to 'F')
    int 0x10        ; Print ASCII character in AL
    dec ch
    jnz .loop       ; Go back if more nibbles to process

    pop ax          ; Restore registers
    pop bx
    pop cx
    pop dx
    pop bp
    ret

; Print string pointed to by DS:SI using
; BIOS TTY output via int 10h/AH=0eh

print_string:
    push ax
    push si
    mov ah, 0Eh       ; int 10h 'print char' function

.repeat:
    lodsb             ; Get character from string
    test al, al
    je .done      ; If char is zero, end of string
    int 10h           ; Otherwise, print it
    jmp .repeat
.done:
    pop si
    pop ax
    ret

; Disk address packet
DAP:
    db 10h, 0
    dw 1 ; Number of sectors to be loaded
    dd bootend
    dq 1

msg_booting:            db "Booting... ", 0x00
err_no_extensions:      db "Int 13h extensions not supported by BIOS", 0x00
err_no_drv_ext_support: db "Int 13h Extensions not supported on drive 0x", 0x00

; Fill the rest of bootsector with zeroes and end it
times 510 - ($ - boot) db 0
dw 0xAA55
bootend:

    mov ax, 0xb800
    mov es, ax
    mov word [es:0x0000], 0x57<<8 | 'M'
    mov word [es:0x0002], 0x57<<8 | 'D'
    mov word [es:0x0004], 0x57<<8 | 'P'
    cli
    hlt